import javax.naming.*;
import javax.naming.directory.*;
import java.util.Hashtable;

/**
  * Demonstrates how to add a new object class to the schema.
  *
  * usage: java AddObjectClass
  */
class AddObjectClass {
    public static void main(String[] args) {

	// Set up environment for creating initial context
	Hashtable env = new Hashtable(11);
	env.put(Context.INITIAL_CONTEXT_FACTORY, 
	    "com.sun.jndi.ldap.LdapCtxFactory");
	env.put(Context.PROVIDER_URL, "ldap://localhost:389/o=JNDITutorial");

	// Must authenticate as directory administrator in order to update schema
	env.put(Context.SECURITY_PRINCIPAL, "cn=Directory Manager");
	env.put(Context.SECURITY_CREDENTIALS, "secret99");

	// These are required for using with Netscape Directory Server 3.x
	env.put(Context.SECURITY_AUTHENTICATION, "simple");
	env.put("com.sun.naming.netscape.schemaBugs", "true");

	try {
	    // Specify attributes for schema object
	    Attributes attrs = new BasicAttributes(true); // ignore case
	    attrs.put("NUMERICOID", "1.3.6.1.4.1.42.2.27.4.2.3.1.1.1");
	    attrs.put("NAME", "fooObjectClass");
	    attrs.put("DESC", "for JNDITutorial example only");
	    attrs.put("SUP", "top");
	    attrs.put("STRUCTURAL", "true");
	    Attribute must = new BasicAttribute("MUST", "cn");
	    must.add("objectclass");
	    attrs.put(must);

	    // Create the initial context
	    DirContext ctx = new InitialDirContext(env);

	    // Get the schema tree root
	    DirContext schema = ctx.getSchema("");

	    // Add new schema object for "fooObjectClass"
    	    DirContext newClass = schema.createSubcontext(
		"ClassDefinition/fooObjectClass", attrs);

	    // See what was added
	    System.out.println(
		schema.getAttributes("ClassDefinition/fooObjectClass"));
	} catch (NamingException e) {
	    e.printStackTrace();
	}
    }
}
